// In diesem Beispiel werden Daten zwischen Processing und Arduino
// über eine UDP-Verbindung ausgetauscht

#include <Servo.h>    // Servo-Library importieren
#include <SPI.h>      // SPI-Library importieren (für Serielle Schnittstelle)
#include <Ethernet.h> // Ethernet-Library importieren (für Ethernet-Schnittstelle)
#include <Udp.h>      // UDP-Library importieren (für Datenübertragung per UDP-Protokol)

int sensorPin = 0;   // Poti-Pin ("Temperaturmessung")
int servoPin = 9;    // Servo-Pin ("Heizungssteuerung")

int servoRot = 0;    // Servo-Rotation (Heizstufe)
int tmpValue = 0;    // Variable zum Speichern der aktuellen "Temperatur"

const char replyBufferLength = 4;  // max. Stellen der zu sendenden Zahlen 
const char receivedDigits = 3;  // max. Stellen der zu empfangenden Zahlen 

Servo servo;         // Servo-Objekt zum Steuern des Servos

//Hub (Intern) Netzwerkeinstellungen
//byte mac[] = { 0x90, 0xA2, 0xDA, 0x00, 0x82, 0xCC }; // MAC-Adresse des Ethernet-Shields
//byte ip[] = { 192, 168, 14, 10 };                    // IP-Adresse des Arduinos
//byte gateway[] = { 192, 168, 14, 254 };              // Netzwerk-Gateway für das interne Netzwerk
//byte subnet[]  = { 255, 255, 255, 0 };               // Subnetz-Maske für das interne Netzwerk

//ZUHAUS Netzwerkeinstellungen
byte mac[] = { 0x90, 0xA2, 0xDA, 0x00, 0x82, 0xCC }; // MAC-Adresse des Ethernet-Shields
byte ip[] = { 192, 168, 178, 222 };                  // IP-Adresse des Arduinos
byte gateway[] = { 192, 168, 178, 1 };               // Netzwerk-Gateway für das interne Netzwerk
byte subnet[]  = { 255, 255, 255, 0 };               // Subnetz-Maske für das interne Netzwerk

unsigned int localPort = 80;  // Port zur Datenübertragung (typischerweise Port 80)

byte remoteIp[4];        //zum Speichern der Adresse des Clients (z.B. Processing)
unsigned int remotePort; //zum Speichern des Ports des Clients (z.B. Processing)

char packetBuffer[UDP_TX_PACKET_MAX_SIZE];  // Puffer zum Empfangen
char ReplyBuffer[replyBufferLength];        // Puffer zum Senden

void setup()
{
  Ethernet.begin(mac,ip, gateway, subnet);  // Ethernet-Verbindung starten
  Udp.begin(localPort);                     // UDP-Verbindung aufbauen

  servo.attach(servoPin);                   // Servo am Servo-Pin aktivieren

  Serial.begin(9600);                       // Serielle Übertragung starten
}
void loop() 
{
  int packetSize = Udp.available();  // potentiell verbundenen Client speichern

  if(packetSize)  // wenn ein Client verbunden ist
  {
    Udp.readPacket(packetBuffer,UDP_TX_PACKET_MAX_SIZE, remoteIp, remotePort); // Die Nachricht im UDP-Paket lesen

    String clientMsg = packetBuffer; // die gelesene Nachricht in der Variable clientMsg speichern

    if (clientMsg.indexOf("getTemperature")>-1) // wenn die Nachricht vom Client den Text "getTemperature" enthält
    {
      tmpValue = analogRead(sensorPin)/2; // vom SensorPin gelesenen Wert durch 2 teilen, weil nur dreistellige Zahlen übertragen werden können
      
      String tmpValueText = (String)tmpValue; // eingelesenen Zahlenwert in String umwandeln
      tmpValueText.toCharArray(ReplyBuffer, replyBufferLength); // Zahlen-String in den Puffer schreiben

      Udp.sendPacket( ReplyBuffer, remoteIp, remotePort); // Inhalt des Puffers an den Client (z.B. Processing) schicken
    }
    
    else if (clientMsg.indexOf("setServoRot")>-1) // wenn die Nachricht vom Client den Text "setServoRot" enthält
    {
      clientMsg = clientMsg.replace("setServoRot", ""); // setServoRot aus der Nachricht löschen, damit nur Zahlenwert übrig bleibt

      char newServoRot [receivedDigits]; // Char-Array zum speichern der übertragenen Nachricht anlegen
      
      for (int i = 0; i < receivedDigits; i++) // Zeichen für Zeichen die Nachricht in das Array übertragen
      {
        newServoRot [i] = clientMsg.charAt(i);
      }
      
      int newServoRotInt = atoi(newServoRot); // Inhalt des Arrays in einen Zahlenwert (Integer) umwandeln
 
      servo.write(newServoRotInt); // den Servo auf die angegebene Position drehen
    }
  }
  delay(15); // 15 Millisekunden warten
}
